@@ -10,7 +10,7 @@ from io import StringIO
 from pathlib import Path
 from textwrap import dedent, indent
 from types import FunctionType, ModuleType
-from typing import (
+from typing import (  # noqa: UP035
     IO,
     Any,
     AnyStr,
@@ -34,6 +34,7 @@ import pytest
 import typing_extensions
 from sphinx.application import Sphinx
 from sphinx.config import Config
+
 from sphinx_autodoc_typehints import (
     _resolve_type_guarded_imports,
     backfill_type_hints,
@@ -71,29 +72,26 @@ P_bound = typing_extensions.ParamSpec("P_bound", bound=str)  # type: ignore[misc
 
 # Mypy does not support recursive type aliases, but
 # other type checkers do.
-RecList = Union[int, List["RecList"]]
-MutualRecA = Union[bool, List["MutualRecB"]]
-MutualRecB = Union[str, List["MutualRecA"]]
+RecList = Union[int, List["RecList"]]  # noqa: UP006
+MutualRecA = Union[bool, List["MutualRecB"]]  # noqa: UP006
+MutualRecB = Union[str, List["MutualRecA"]]  # noqa: UP006
 
 
 class A:
     def get_type(self) -> type:
         return type(self)
 
-    class Inner:
-        ...
+    class Inner: ...
 
 
 class B(Generic[T]):
     name = "Foo"  # This is set to make sure the correct class name ("B") is picked up
 
 
-class C(B[str]):
-    ...
+class C(B[str]): ...
 
 
-class D(typing_extensions.Protocol):
-    ...
+class D(typing_extensions.Protocol): ...
 
 
 class E(typing_extensions.Protocol[T]):  # type: ignore[misc]
@@ -104,8 +102,7 @@ class Slotted:
     __slots__ = ()
 
 
-class Metaclass(type):
-    ...
+class Metaclass(type): ...
 
 
 class HintedMethods:
@@ -120,7 +117,7 @@ class HintedMethods:
 PY310_PLUS = sys.version_info >= (3, 10)
 PY312_PLUS = sys.version_info >= (3, 12)
 
-if sys.version_info >= (3, 9):
+if sys.version_info >= (3, 9):  # noqa: UP036
     AbcCallable = collections.abc.Callable  # type: ignore[type-arg]
 else:
     # We could also set AbcCallable = typing.Callable and x fail the tests that
@@ -148,11 +145,11 @@ else:
         pytest.param(types.CoroutineType, "types", "CoroutineType", (), id="CoroutineType"),
         pytest.param(Any, "typing", "Any", (), id="Any"),
         pytest.param(AnyStr, "typing", "AnyStr", (), id="AnyStr"),
-        pytest.param(Dict, "typing", "Dict", (), id="Dict"),
-        pytest.param(Dict[str, int], "typing", "Dict", (str, int), id="Dict_parametrized"),
-        pytest.param(Dict[T, int], "typing", "Dict", (T, int), id="Dict_typevar"),  # type: ignore[valid-type]
-        pytest.param(Tuple, "typing", "Tuple", (), id="Tuple"),
-        pytest.param(Tuple[str, int], "typing", "Tuple", (str, int), id="Tuple_parametrized"),
+        pytest.param(Dict, "typing", "Dict", (), id="Dict"),  # noqa: UP006
+        pytest.param(Dict[str, int], "typing", "Dict", (str, int), id="Dict_parametrized"),  # noqa: UP006
+        pytest.param(Dict[T, int], "typing", "Dict", (T, int), id="Dict_typevar"),  # type: ignore[valid-type]  # noqa: UP006
+        pytest.param(Tuple, "typing", "Tuple", (), id="Tuple"),  # noqa: UP006
+        pytest.param(Tuple[str, int], "typing", "Tuple", (str, int), id="Tuple_parametrized"),  # noqa: UP006
         pytest.param(Union[str, int], "typing", "Union", (str, int), id="Union"),
         pytest.param(Callable, "typing", "Callable", (), id="Callable"),
         pytest.param(Callable[..., str], "typing", "Callable", (..., str), id="Callable_returntype"),
@@ -192,131 +189,191 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
 
 
 _CASES = [
-    (str, ":py:class:`str`"),
-    (int, ":py:class:`int`"),
-    (StringIO, ":py:class:`~io.StringIO`"),
-    (FunctionType, ":py:class:`~types.FunctionType`"),
-    (ModuleType, ":py:class:`~types.ModuleType`"),
-    (type(None), ":py:obj:`None`"),
-    (type, ":py:class:`type`"),
-    (collections.abc.Callable, ":py:class:`~collections.abc.Callable`"),
-    (Type, ":py:class:`~typing.Type`"),
-    (Type[A], r":py:class:`~typing.Type`\ \[:py:class:`~%s.A`]" % __name__),
-    (Any, ":py:data:`~typing.Any`"),
-    (AnyStr, ":py:data:`~typing.AnyStr`"),
-    (Generic[T], r":py:class:`~typing.Generic`\ \[:py:class:`~typing.TypeVar`\ \(``T``)]"),
-    (Mapping, ":py:class:`~typing.Mapping`"),
-    (
+    pytest.param(str, ":py:class:`str`", id="str"),
+    pytest.param(int, ":py:class:`int`", id="int"),
+    pytest.param(StringIO, ":py:class:`~io.StringIO`", id="StringIO"),
+    pytest.param(FunctionType, ":py:class:`~types.FunctionType`", id="FunctionType"),
+    pytest.param(ModuleType, ":py:class:`~types.ModuleType`", id="ModuleType"),
+    pytest.param(type(None), ":py:obj:`None`", id="type None"),
+    pytest.param(type, ":py:class:`type`", id="type"),
+    pytest.param(collections.abc.Callable, ":py:class:`~collections.abc.Callable`", id="abc-Callable"),
+    pytest.param(Type, ":py:class:`~typing.Type`", id="typing-Type"),  # noqa: UP006
+    pytest.param(Type[A], rf":py:class:`~typing.Type`\ \[:py:class:`~{__name__}.A`]", id="typing-A"),  # noqa: UP006
+    pytest.param(Any, ":py:data:`~typing.Any`", id="Any"),
+    pytest.param(AnyStr, ":py:data:`~typing.AnyStr`", id="AnyStr"),
+    pytest.param(Generic[T], r":py:class:`~typing.Generic`\ \[:py:class:`~typing.TypeVar`\ \(``T``)]", id="Generic"),
+    pytest.param(Mapping, ":py:class:`~typing.Mapping`", id="Mapping"),
+    pytest.param(
         Mapping[T, int],  # type: ignore[valid-type]
         r":py:class:`~typing.Mapping`\ \[:py:class:`~typing.TypeVar`\ \(``T``), :py:class:`int`]",
+        id="Mapping-T-int",
     ),
-    (
+    pytest.param(
         Mapping[str, V_contra],  # type: ignore[valid-type]
         r":py:class:`~typing.Mapping`\ \[:py:class:`str`, :py:class:`~typing.TypeVar`\ \("
         "``V_contra``, contravariant=True)]",
+        id="Mapping-T-int-contra",
     ),
-    (
+    pytest.param(
         Mapping[T, U_co],  # type: ignore[valid-type]
         r":py:class:`~typing.Mapping`\ \[:py:class:`~typing.TypeVar`\ \(``T``), "
         r":py:class:`~typing.TypeVar`\ \(``U_co``, covariant=True)]",
+        id="Mapping-T-int-co",
     ),
-    (Mapping[str, bool], r":py:class:`~typing.Mapping`\ \[:py:class:`str`, :py:class:`bool`]"),
-    (Dict, ":py:class:`~typing.Dict`"),
-    (
-        Dict[T, int],  # type: ignore[valid-type]
+    pytest.param(
+        Mapping[str, bool],
+        r":py:class:`~typing.Mapping`\ \[:py:class:`str`, :py:class:`bool`]",
+        id="Mapping-str-bool",
+    ),
+    pytest.param(Dict, ":py:class:`~typing.Dict`", id="Dict"),  # noqa: UP006
+    pytest.param(
+        Dict[T, int],  # type: ignore[valid-type]  # noqa: UP006
         r":py:class:`~typing.Dict`\ \[:py:class:`~typing.TypeVar`\ \(``T``), :py:class:`int`]",
+        id="Dict-T-int",
     ),
-    (
-        Dict[str, V_contra],  # type: ignore[valid-type]
-        r":py:class:`~typing.Dict`\ \[:py:class:`str`, :py:class:`~typing.TypeVar`\ \(``V_contra``, contravariant=True)]",  # noqa: E501
+    pytest.param(
+        Dict[str, V_contra],  # type: ignore[valid-type]  # noqa: UP006
+        r":py:class:`~typing.Dict`\ \[:py:class:`str`, :py:class:`~typing.TypeVar`\ \(``V_contra``, "
+        r"contravariant=True)]",
+        id="Dict-T-int-contra",
     ),
-    (
-        Dict[T, U_co],  # type: ignore[valid-type]
+    pytest.param(
+        Dict[T, U_co],  # type: ignore[valid-type]  # noqa: UP006
         r":py:class:`~typing.Dict`\ \[:py:class:`~typing.TypeVar`\ \(``T``),"
         r" :py:class:`~typing.TypeVar`\ \(``U_co``, covariant=True)]",
+        id="Dict-T-int-co",
     ),
-    (Dict[str, bool], r":py:class:`~typing.Dict`\ \[:py:class:`str`, :py:class:`bool`]"),
-    (Tuple, ":py:data:`~typing.Tuple`"),
-    (Tuple[str, bool], r":py:data:`~typing.Tuple`\ \[:py:class:`str`, :py:class:`bool`]"),
-    (Tuple[int, int, int], r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:class:`int`, :py:class:`int`]"),
-    (Tuple[str, ...], r":py:data:`~typing.Tuple`\ \[:py:class:`str`, :py:data:`...<Ellipsis>`]"),
-    (Union, ":py:data:`~typing.Union`"),
-    (Union[str, bool], r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:class:`bool`]"),
-    (Union[str, bool, None], r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:class:`bool`, :py:obj:`None`]"),
-    pytest.param(Union[str, Any], r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:data:`~typing.Any`]"),
-    (Optional[str], r":py:data:`~typing.Optional`\ \[:py:class:`str`]"),
-    (Union[str, None], r":py:data:`~typing.Optional`\ \[:py:class:`str`]"),
-    (
+    pytest.param(
+        Dict[str, bool],  # noqa: UP006
+        r":py:class:`~typing.Dict`\ \[:py:class:`str`, :py:class:`bool`]",
+        id="Dict-str-bool",  # noqa: RUF100, UP006
+    ),
+    pytest.param(Tuple, ":py:data:`~typing.Tuple`", id="Tuple"),  # noqa: UP006
+    pytest.param(
+        Tuple[str, bool],  # noqa: UP006
+        r":py:data:`~typing.Tuple`\ \[:py:class:`str`, :py:class:`bool`]",
+        id="Tuple-str-bool",  # noqa: RUF100, UP006
+    ),
+    pytest.param(
+        Tuple[int, int, int],  # noqa: UP006
+        r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:class:`int`, :py:class:`int`]",
+        id="Tuple-int-int-int",
+    ),
+    pytest.param(
+        Tuple[str, ...],  # noqa: UP006
+        r":py:data:`~typing.Tuple`\ \[:py:class:`str`, :py:data:`...<Ellipsis>`]",
+        id="Tuple-str-Ellipsis",
+    ),
+    pytest.param(Union, ":py:data:`~typing.Union`", id="Union"),
+    pytest.param(
+        Union[str, bool], r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:class:`bool`]", id="Union-str-bool"
+    ),
+    pytest.param(
+        Union[str, bool, None],
+        r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:class:`bool`, :py:obj:`None`]",
+        id="Union-str-bool-None",
+    ),
+    pytest.param(
+        Union[str, Any], r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:data:`~typing.Any`]", id="Union-str-Any"
+    ),
+    pytest.param(Optional[str], r":py:data:`~typing.Optional`\ \[:py:class:`str`]", id="Optional-str"),
+    pytest.param(Union[str, None], r":py:data:`~typing.Optional`\ \[:py:class:`str`]", id="Optional-str-None"),
+    pytest.param(
         Optional[Union[str, bool]],
         r":py:data:`~typing.Union`\ \[:py:class:`str`, :py:class:`bool`, :py:obj:`None`]",
+        id="Optional-Union-str-bool",
     ),
-    (Callable, ":py:data:`~typing.Callable`"),
-    (Callable[..., int], r":py:data:`~typing.Callable`\ \[:py:data:`...<Ellipsis>`, :py:class:`int`]"),
-    (Callable[[int], int], r":py:data:`~typing.Callable`\ \[\[:py:class:`int`], :py:class:`int`]"),
-    (
+    pytest.param(Callable, ":py:data:`~typing.Callable`", id="Callable"),
+    pytest.param(
+        Callable[..., int],
+        r":py:data:`~typing.Callable`\ \[:py:data:`...<Ellipsis>`, :py:class:`int`]",
+        id="Callable-Ellipsis-int",
+    ),
+    pytest.param(
+        Callable[[int], int],
+        r":py:data:`~typing.Callable`\ \[\[:py:class:`int`], :py:class:`int`]",
+        id="Callable-int-int",
+    ),
+    pytest.param(
         Callable[[int, str], bool],
         r":py:data:`~typing.Callable`\ \[\[:py:class:`int`, :py:class:`str`], :py:class:`bool`]",
+        id="Callable-int-str-bool",
     ),
-    (
+    pytest.param(
         Callable[[int, str], None],
         r":py:data:`~typing.Callable`\ \[\[:py:class:`int`, :py:class:`str`], :py:obj:`None`]",
+        id="Callable-int-str",
     ),
-    (
+    pytest.param(
         Callable[[T], T],
         r":py:data:`~typing.Callable`\ \[\[:py:class:`~typing.TypeVar`\ \(``T``)],"
         r" :py:class:`~typing.TypeVar`\ \(``T``)]",
+        id="Callable-T-T",
     ),
-    (
+    pytest.param(
         AbcCallable[[int, str], bool],  # type: ignore[valid-type,misc,type-arg]
         r":py:class:`~collections.abc.Callable`\ \[\[:py:class:`int`, :py:class:`str`], :py:class:`bool`]",
+        id="AbcCallable-int-str-bool",
     ),
-    (Pattern, ":py:class:`~typing.Pattern`"),
-    (Pattern[str], r":py:class:`~typing.Pattern`\ \[:py:class:`str`]"),
-    (IO, ":py:class:`~typing.IO`"),
-    (IO[str], r":py:class:`~typing.IO`\ \[:py:class:`str`]"),
-    (Metaclass, ":py:class:`~%s.Metaclass`" % __name__),
-    (A, ":py:class:`~%s.A`" % __name__),
-    (B, ":py:class:`~%s.B`" % __name__),
-    (B[int], r":py:class:`~%s.B`\ \[:py:class:`int`]" % __name__),
-    (C, ":py:class:`~%s.C`" % __name__),
-    (D, ":py:class:`~%s.D`" % __name__),
-    (E, ":py:class:`~%s.E`" % __name__),
-    (E[int], r":py:class:`~%s.E`\ \[:py:class:`int`]" % __name__),
-    (W, rf":py:{'class' if PY310_PLUS else 'func'}:`~typing.NewType`\ \(``W``, :py:class:`str`)"),
-    (T, r":py:class:`~typing.TypeVar`\ \(``T``)"),
-    (U_co, r":py:class:`~typing.TypeVar`\ \(``U_co``, covariant=True)"),
-    (V_contra, r":py:class:`~typing.TypeVar`\ \(``V_contra``, contravariant=True)"),
-    (X, r":py:class:`~typing.TypeVar`\ \(``X``, :py:class:`str`, :py:class:`int`)"),
-    (Y, r":py:class:`~typing.TypeVar`\ \(``Y``, bound= :py:class:`str`)"),
-    (Z, r":py:class:`~typing.TypeVar`\ \(``Z``, bound= A)"),
-    (S, r":py:class:`~typing.TypeVar`\ \(``S``, bound= miss)"),
+    pytest.param(Pattern, ":py:class:`~typing.Pattern`", id="Pattern"),
+    pytest.param(Pattern[str], r":py:class:`~typing.Pattern`\ \[:py:class:`str`]", id="Pattern-str"),
+    pytest.param(IO, ":py:class:`~typing.IO`", id="IO"),
+    pytest.param(IO[str], r":py:class:`~typing.IO`\ \[:py:class:`str`]", id="IO-str"),
+    pytest.param(Metaclass, f":py:class:`~{__name__}.Metaclass`", id="Metaclass"),
+    pytest.param(A, f":py:class:`~{__name__}.A`", id="A"),
+    pytest.param(B, f":py:class:`~{__name__}.B`", id="B"),
+    pytest.param(B[int], rf":py:class:`~{__name__}.B`\ \[:py:class:`int`]", id="B-int"),
+    pytest.param(C, f":py:class:`~{__name__}.C`", id="C"),
+    pytest.param(D, f":py:class:`~{__name__}.D`", id="D"),
+    pytest.param(E, f":py:class:`~{__name__}.E`", id="E"),
+    pytest.param(E[int], rf":py:class:`~{__name__}.E`\ \[:py:class:`int`]", id="E-int"),
+    pytest.param(W, rf":py:{'class' if PY310_PLUS else 'func'}:`~typing.NewType`\ \(``W``, :py:class:`str`)", id="W"),
+    pytest.param(T, r":py:class:`~typing.TypeVar`\ \(``T``)", id="T"),
+    pytest.param(U_co, r":py:class:`~typing.TypeVar`\ \(``U_co``, covariant=True)", id="U-co"),
+    pytest.param(V_contra, r":py:class:`~typing.TypeVar`\ \(``V_contra``, contravariant=True)", id="V-contra"),
+    pytest.param(X, r":py:class:`~typing.TypeVar`\ \(``X``, :py:class:`str`, :py:class:`int`)", id="X"),
+    pytest.param(Y, r":py:class:`~typing.TypeVar`\ \(``Y``, bound= :py:class:`str`)", id="Y"),
+    pytest.param(Z, r":py:class:`~typing.TypeVar`\ \(``Z``, bound= A)", id="Z"),
+    pytest.param(S, r":py:class:`~typing.TypeVar`\ \(``S``, bound= miss)", id="S"),
     # ParamSpec should behave like TypeVar, except for missing constraints
-    (P, rf":py:class:`~typing.ParamSpec`\ \(``P``{', bound= :py:obj:`None`' if PY312_PLUS else ''})"),
-    (
+    pytest.param(
+        P, rf":py:class:`~typing.ParamSpec`\ \(``P``{', bound= :py:obj:`None`' if PY312_PLUS else ''})", id="P"
+    ),
+    pytest.param(
         P_co,
         rf":py:class:`~typing.ParamSpec`\ \(``P_co``{', bound= :py:obj:`None`' if PY312_PLUS else ''}, covariant=True)",
+        id="P_co",
     ),
-    (
+    pytest.param(
         P_contra,
         rf":py:class:`~typing.ParamSpec`\ \(``P_contra``{', bound= :py:obj:`None`' if PY312_PLUS else ''}"
         ", contravariant=True)",
+        id="P-contra",
     ),
-    (P_bound, r":py:class:`~typing.ParamSpec`\ \(``P_bound``, bound= :py:class:`str`)"),
+    pytest.param(P_bound, r":py:class:`~typing.ParamSpec`\ \(``P_bound``, bound= :py:class:`str`)", id="P-bound"),
     # ## These test for correct internal tuple rendering, even if not all are valid Tuple types
     # Zero-length tuple remains
-    (Tuple[()], ":py:data:`~typing.Tuple`"),
+    pytest.param(Tuple[()], ":py:data:`~typing.Tuple`", id="Tuple-p"),  # noqa: UP006
     # Internal single tuple with simple types is flattened in the output
-    (Tuple[(int,)], r":py:data:`~typing.Tuple`\ \[:py:class:`int`]"),
-    (Tuple[(int, int)], r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:class:`int`]"),
-    # Ellipsis in single tuple also gets flattened
-    (Tuple[(int, ...)], r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:data:`...<Ellipsis>`]"),
-    (
-        RecList,
-        r":py:data:`~typing.Union`\ \[:py:class:`int`, :py:class:`~typing.List`\ \[RecList]]",
+    pytest.param(Tuple[(int,)], r":py:data:`~typing.Tuple`\ \[:py:class:`int`]", id="Tuple-p-int"),  # noqa: UP006
+    pytest.param(
+        Tuple[(int, int)],  # noqa: UP006
+        r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:class:`int`]",
+        id="Tuple-p-int-int",  # noqa: RUF100, UP006
     ),
-    (
+    # Ellipsis in single tuple also gets flattened
+    pytest.param(
+        Tuple[(int, ...)],  # noqa: UP006
+        r":py:data:`~typing.Tuple`\ \[:py:class:`int`, :py:data:`...<Ellipsis>`]",
+        id="Tuple-p-Ellipsis",
+    ),
+    pytest.param(
+        RecList, r":py:data:`~typing.Union`\ \[:py:class:`int`, :py:class:`~typing.List`\ \[RecList]]", id="RecList"
+    ),
+    pytest.param(
         MutualRecA,
         r":py:data:`~typing.Union`\ \[:py:class:`bool`, :py:class:`~typing.List`\ \[MutualRecB]]",
+        id="MutualRecA",
     ),
 ]
 
@@ -324,44 +381,50 @@ if nptyping is not None:
     _CASES.extend(
         [  # Internal tuple with following additional type cannot be flattened (specific to nptyping?)
             # These cases will fail if nptyping restructures its internal module hierarchy
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["*"], nptyping.Float],
                 (
                     ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:class:`~nptyping.base_meta_classes.Shape`\\ \\[*], "
                     ":py:class:`~numpy.float64`]"
                 ),
+                id="NDArray-star-float",
             ),
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["64"], nptyping.Float],
                 (
                     ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:class:`~nptyping.base_meta_classes.Shape`\\ \\[64],"
                     " :py:class:`~numpy.float64`]"
                 ),
+                id="NDArray-64-float",
             ),
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["*, *"], nptyping.Float],
                 (
                     ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:class:`~nptyping.base_meta_classes.Shape`\\ \\[*, "
                     "*], :py:class:`~numpy.float64`]"
                 ),
+                id="NDArray-star-star-float",
             ),
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["*, ..."], nptyping.Float],
                 ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:data:`~typing.Any`, :py:class:`~numpy.float64`]",
+                id="NDArray-star-Ellipsis-float",
             ),
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["*, 3"], nptyping.Float],
                 (
                     ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:class:`~nptyping.base_meta_classes.Shape`\\ \\[*, 3"
                     "], :py:class:`~numpy.float64`]"
                 ),
+                id="NDArray-star-3-float",
             ),
-            (
+            pytest.param(
                 nptyping.NDArray[nptyping.Shape["3, ..."], nptyping.Float],
                 (
                     ":py:class:`~nptyping.ndarray.NDArray`\\ \\[:py:class:`~nptyping.base_meta_classes.Shape`\\ \\[3, "
                     "...], :py:class:`~numpy.float64`]"
                 ),
+                id="NDArray-3-Ellipsis-float",
             ),
         ],
     )
@@ -369,7 +432,7 @@ if nptyping is not None:
 
 @pytest.mark.parametrize(("annotation", "expected_result"), _CASES)
 def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str) -> None:
-    conf = create_autospec(Config, _annotation_globals=globals())
+    conf = create_autospec(Config, _annotation_globals=globals(), always_use_bars_union=False)
     result = format_annotation(annotation, conf)
     assert result == expected_result
 
@@ -379,9 +442,14 @@ def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str
         # subsequent tests
         expected_result_not_simplified = expected_result.replace(", ``None``", "")
         # encapsulate Union in typing.Optional
-        expected_result_not_simplified = ":py:data:`~typing.Optional`\\ \\[" + expected_result_not_simplified
+        expected_result_not_simplified += ":py:data:`~typing.Optional`\\ \\["
         expected_result_not_simplified += "]"
-        conf = create_autospec(Config, simplify_optional_unions=False, _annotation_globals=globals())
+        conf = create_autospec(
+            Config,
+            simplify_optional_unions=False,
+            _annotation_globals=globals(),
+            always_use_bars_union=False,
+        )
         assert format_annotation(annotation, conf) == expected_result_not_simplified
 
         # Test with the "fully_qualified" flag turned on
@@ -401,7 +469,12 @@ def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str
         expected_result = expected_result.replace("~nptyping", "nptyping")
         expected_result = expected_result.replace("~numpy", "numpy")
         expected_result = expected_result.replace("~" + __name__, __name__)
-        conf = create_autospec(Config, typehints_fully_qualified=True, _annotation_globals=globals())
+        conf = create_autospec(
+            Config,
+            typehints_fully_qualified=True,
+            _annotation_globals=globals(),
+            always_use_bars_union=False,
+        )
         assert format_annotation(annotation, conf) == expected_result
 
     # Test for the correct role (class vs data) using the official Sphinx inventory
@@ -417,15 +490,35 @@ def test_format_annotation(inv: Inventory, annotation: Any, expected_result: str
             assert m.group("role") == expected_role
 
 
+@pytest.mark.parametrize(
+    ("annotation", "expected_result"),
+    [
+        ("int | float", ":py:class:`int` | :py:class:`float`"),
+        ("int | float | None", ":py:class:`int` | :py:class:`float` | :py:obj:`None`"),
+        ("Union[int, float]", ":py:class:`int` | :py:class:`float`"),
+        ("Union[int, float, None]", ":py:class:`int` | :py:class:`float` | :py:obj:`None`"),
+        ("Optional[int | float]", ":py:class:`int` | :py:class:`float` | :py:obj:`None`"),
+        ("Optional[Union[int, float]]", ":py:class:`int` | :py:class:`float` | :py:obj:`None`"),
+        ("Union[int | float, str]", ":py:class:`int` | :py:class:`float` | :py:class:`str`"),
+        ("Union[int, float] | str", ":py:class:`int` | :py:class:`float` | :py:class:`str`"),
+    ],
+)
+@pytest.mark.skipif(not PY310_PLUS, reason="| union doesn't work before py310")
+def test_always_use_bars_union(annotation: str, expected_result: str) -> None:
+    conf = create_autospec(Config, always_use_bars_union=True)
+    result = format_annotation(eval(annotation), conf)  # noqa: S307
+    assert result == expected_result
+
+
 @pytest.mark.parametrize("library", [typing, typing_extensions], ids=["typing", "typing_extensions"])
 @pytest.mark.parametrize(
     ("annotation", "params", "expected_result"),
     [
-        ("ClassVar", int, ":py:data:`~typing.ClassVar`\\ \\[:py:class:`int`]"),
-        ("NoReturn", None, ":py:data:`~typing.NoReturn`"),
-        ("Literal", ("a", 1), ":py:data:`~typing.Literal`\\ \\[``'a'``, ``1``]"),
-        ("Type", None, ":py:class:`~typing.Type`"),
-        ("Type", (A,), rf":py:class:`~typing.Type`\ \[:py:class:`~{__name__}.A`]"),
+        pytest.param("ClassVar", int, ":py:data:`~typing.ClassVar`\\ \\[:py:class:`int`]", id="ClassVar"),
+        pytest.param("NoReturn", None, ":py:data:`~typing.NoReturn`", id="NoReturn"),
+        pytest.param("Literal", ("a", 1), ":py:data:`~typing.Literal`\\ \\[``'a'``, ``1``]", id="Literal"),
+        pytest.param("Type", None, ":py:class:`~typing.Type`", id="Type-none"),
+        pytest.param("Type", (A,), rf":py:class:`~typing.Type`\ \[:py:class:`~{__name__}.A`]", id="Type-A"),
     ],
 )
 def test_format_annotation_both_libs(library: ModuleType, annotation: str, params: Any, expected_result: str) -> None:
@@ -472,8 +565,8 @@ def test_always_document_param_types(
 ) -> None:
     set_python_path()
 
-    app.config.always_document_param_types = always_document_param_types  # type: ignore[attr-defined] # create flag
-    app.config.autodoc_mock_imports = ["mailbox"]  # type: ignore[attr-defined] # create flag
+    app.config.always_document_param_types = always_document_param_types  # create flag
+    app.config.autodoc_mock_imports = ["mailbox"]  # create flag
 
     # Prevent "document isn't included in any toctree" warnings
     for f in Path(app.srcdir).glob("*.rst"):
@@ -499,7 +592,7 @@ def test_always_document_param_types(
     for indentation_level in range(2):
         key = f"undoc_params_{indentation_level}"
         if always_document_param_types:
-            format_args[key] = indent('\n\n   Parameters:\n      **x** ("int") --', "   " * indentation_level)
+            format_args[key] = indent('\n\n   Parameters:\n      **x** ("int")', "   " * indentation_level)
         else:
             format_args[key] = ""
 
@@ -523,12 +616,13 @@ def test_always_document_param_types(
 
 
 def maybe_fix_py310(expected_contents: str) -> str:
+    if sys.version_info >= (3, 11):
+        return expected_contents
     if not PY310_PLUS:
         return expected_contents.replace('"', "")
 
     for old, new in [
-        ("bool | None", '"Optional"["bool"]'),
-        ("str | None", '"Optional"["str"]'),
+        ('"str" | "None"', '"Optional"["str"]'),
     ]:
         expected_contents = expected_contents.replace(old, new)
     return expected_contents
@@ -539,7 +633,7 @@ def maybe_fix_py310(expected_contents: str) -> str:
 def test_sphinx_output_future_annotations(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
 
-    app.config.master_doc = "future_annotations"  # type: ignore[attr-defined] # create flag
+    app.config.master_doc = "future_annotations"  # create flag
     app.build()
 
     assert "build succeeded" in status.getvalue()  # Build succeeded
@@ -554,15 +648,16 @@ def test_sphinx_output_future_annotations(app: SphinxTestApp, status: StringIO)
        Method docstring.
 
        Parameters:
-          * **x** (bool | None) -- foo
+          * **x** ("bool" | "None") -- foo
 
           * **y** ("int" | "str" | "float") -- bar
 
-          * **z** (str | None) -- baz
+          * **z** ("str" | "None") -- baz
 
        Return type:
           "str"
     """
+    expected_contents = dedent(expected_contents)
     expected_contents = maybe_fix_py310(dedent(expected_contents))
     assert contents == expected_contents
 
@@ -572,13 +667,15 @@ def test_sphinx_output_future_annotations(app: SphinxTestApp, status: StringIO)
 def test_sphinx_output_default_role(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
 
-    app.config.master_doc = "simple_default_role"  # type: ignore[attr-defined] # create flag
-    app.config.default_role = "literal"  # type: ignore[attr-defined]
+    app.config.master_doc = "simple_default_role"  # create flag
+    app.config.default_role = "literal"
     app.build()
 
     assert "build succeeded" in status.getvalue()  # Build succeeded
 
-    contents_lines = (Path(app.srcdir) / "_build/pseudoxml/simple_default_role.pseudoxml").read_text().splitlines()
+    contents_lines = (
+        (Path(app.srcdir) / "_build/pseudoxml/simple_default_role.pseudoxml").read_text(encoding="utf-8").splitlines()
+    )
     list_item_idxs = [i for i, line in enumerate(contents_lines) if line.strip() == "<list_item>"]
     foo_param = dedent("\n".join(contents_lines[list_item_idxs[0] : list_item_idxs[1]]))
     expected_foo_param = """\
@@ -619,8 +716,8 @@ def test_sphinx_output_defaults(
 ) -> None:
     set_python_path()
 
-    app.config.master_doc = "simple"  # type: ignore[attr-defined] # create flag
-    app.config.typehints_defaults = defaults_config_val  # type: ignore[attr-defined] # create flag
+    app.config.master_doc = "simple"  # create flag
+    app.config.typehints_defaults = defaults_config_val  # create flag
     if isinstance(expected, Exception):
         with pytest.raises(Exception, match=re.escape(str(expected))):
             app.build()
@@ -666,8 +763,8 @@ def test_sphinx_output_formatter(
 ) -> None:
     set_python_path()
 
-    app.config.master_doc = "simple"  # type: ignore[attr-defined] # create flag
-    app.config.typehints_formatter = formatter_config_val  # type: ignore[attr-defined] # create flag
+    app.config.master_doc = "simple"  # create flag
+    app.config.typehints_formatter = formatter_config_val  # create flag
     if isinstance(expected, Exception):
         with pytest.raises(Exception, match=re.escape(str(expected))):
             app.build()
@@ -783,6 +880,7 @@ def test_syntax_error_backfill() -> None:
     # fmt: off
     def func(x):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN202
         return x
+
     # fmt: on
     backfill_type_hints(func, "func")
 
@@ -790,7 +888,7 @@ def test_syntax_error_backfill() -> None:
 @pytest.mark.sphinx("text", testroot="resolve-typing-guard")
 def test_resolve_typing_guard_imports(app: SphinxTestApp, status: StringIO, warning: StringIO) -> None:
     set_python_path()
-    app.config.autodoc_mock_imports = ["viktor"]  # type: ignore[attr-defined] # create flag
+    app.config.autodoc_mock_imports = ["viktor"]  # create flag
     app.build()
     out = status.getvalue()
     assert "build succeeded" in out
@@ -810,7 +908,7 @@ def test_resolve_typing_guard_attrs_imports(app: SphinxTestApp, status: StringIO
 
 
 def test_no_source_code_type_guard() -> None:
-    from csv import Error
+    from csv import Error  # noqa: PLC0415
 
     _resolve_type_guarded_imports([], Error)
 
@@ -819,8 +917,8 @@ def test_no_source_code_type_guard() -> None:
 @patch("sphinx.writers.text.MAXWIDTH", 2000)
 def test_sphinx_output_formatter_no_use_rtype(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
-    app.config.master_doc = "simple_no_use_rtype"  # type: ignore[attr-defined]  # create flag
-    app.config.typehints_use_rtype = False  # type: ignore[attr-defined]
+    app.config.master_doc = "simple_no_use_rtype"  # create flag
+    app.config.typehints_use_rtype = False
     app.build()
     assert "build succeeded" in status.getvalue()
     text_path = Path(app.srcdir) / "_build" / "text" / "simple_no_use_rtype.txt"
@@ -884,8 +982,8 @@ def test_sphinx_output_formatter_no_use_rtype(app: SphinxTestApp, status: String
 @patch("sphinx.writers.text.MAXWIDTH", 2000)
 def test_sphinx_output_with_use_signature(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
-    app.config.master_doc = "simple"  # type: ignore[attr-defined] # create flag
-    app.config.typehints_use_signature = True  # type: ignore[attr-defined]
+    app.config.master_doc = "simple"  # create flag
+    app.config.typehints_use_signature = True
     app.build()
     assert "build succeeded" in status.getvalue()
     text_path = Path(app.srcdir) / "_build" / "text" / "simple.txt"
@@ -913,8 +1011,8 @@ def test_sphinx_output_with_use_signature(app: SphinxTestApp, status: StringIO)
 @patch("sphinx.writers.text.MAXWIDTH", 2000)
 def test_sphinx_output_with_use_signature_return(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
-    app.config.master_doc = "simple"  # type: ignore[attr-defined] # create flag
-    app.config.typehints_use_signature_return = True  # type: ignore[attr-defined]
+    app.config.master_doc = "simple"  # create flag
+    app.config.typehints_use_signature_return = True
     app.build()
     assert "build succeeded" in status.getvalue()
     text_path = Path(app.srcdir) / "_build" / "text" / "simple.txt"
@@ -942,9 +1040,9 @@ def test_sphinx_output_with_use_signature_return(app: SphinxTestApp, status: Str
 @patch("sphinx.writers.text.MAXWIDTH", 2000)
 def test_sphinx_output_with_use_signature_and_return(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
-    app.config.master_doc = "simple"  # type: ignore[attr-defined] # create flag
-    app.config.typehints_use_signature = True  # type: ignore[attr-defined]
-    app.config.typehints_use_signature_return = True  # type: ignore[attr-defined]
+    app.config.master_doc = "simple"  # create flag
+    app.config.typehints_use_signature = True
+    app.config.typehints_use_signature_return = True
     app.build()
     assert "build succeeded" in status.getvalue()
     text_path = Path(app.srcdir) / "_build" / "text" / "simple.txt"
@@ -972,8 +1070,8 @@ def test_sphinx_output_with_use_signature_and_return(app: SphinxTestApp, status:
 @patch("sphinx.writers.text.MAXWIDTH", 2000)
 def test_default_annotation_without_typehints(app: SphinxTestApp, status: StringIO) -> None:
     set_python_path()
-    app.config.master_doc = "without_complete_typehints"  # type: ignore[attr-defined]# create flag
-    app.config.typehints_defaults = "comma"  # type: ignore[attr-defined]
+    app.config.master_doc = "without_complete_typehints"  # create flag
+    app.config.typehints_defaults = "comma"
     app.build()
     assert "build succeeded" in status.getvalue()
     text_path = Path(app.srcdir) / "_build" / "text" / "without_complete_typehints.txt"
